<?php

/**
 * @package "Image Watermark" Addon for Elkarte
 * @author Spuds
 * @copyright (c) 2022 Spuds
 * @license This Source Code is subject to the terms of the Mozilla Public License
 * version 1.1 (the "License"). You can obtain a copy of the License at
 * http://mozilla.org/MPL/1.1/.
 *
 * @version 1.0.1
 *
 */

/**
 * If the file is an image and one that we should process, then it
 * decides which watermark function to call. If successful will
 * output the image and end with obExit
 *
 * @param string $filename
 * @param string $mime_type
 * @param bool $output if to return or directly output the image
 * @return bool;
 */
function processWatermark($filename, $mime_type, $output = true)
{
	global $modSettings;

	$result = false;

	// Not an image
	if ($mime_type === '' || strpos($mime_type, 'image/') !== 0)
	{
		return false;
	}

	// Not a valid or accessible file
	if (!validateWatermark($filename))
	{
		return false;
	}

	// Text or image watermark
	if (!empty($modSettings['watermarkImage']))
	{
		$result = watermark($filename, BOARDDIR . '/themes/default/images/watermark/' . $modSettings['watermarkImageFile']);
	}
	elseif (!empty($modSettings['watermarkText']))
	{
		$result = watermarkText($filename);
	}

	// Success, we hope
	if ($result !== false)
	{
		// Keep the results the same as we were given
		if (!empty($mime_type))
		{
			$mime_type = str_replace('image/', '', $mime_type);
			$result->setImageFormat($mime_type);
		}

		if ($output)
		{
			echo $result->getImageBlob();
			obExit(false);
		}
	}

	return $result;
}

/**
 * Reads the watermark image directory to find all image files we can use
 *
 * @return array
 */
function watermarkDiscoverImages()
{
	global $settings;

	$allowedTypes = addWebpType(array('png', 'jpg', 'jpeg'));
	$directory = $settings['default_theme_dir'] . '/images/watermark';
	$watermarkImages = array(' ' => '');

	// Well this is easy
	if (!is_dir($directory))
	{
		return $watermarkImages;
	}

	// Otherwise we check for files
	$fileSystemIterator = new FilesystemIterator($directory, FilesystemIterator::SKIP_DOTS);
	foreach ($fileSystemIterator as $fileInfo)
	{
		if ($fileInfo->getFilename() !== 'admin_example.jpg'
			&& in_array(strtolower($fileInfo->getExtension()), $allowedTypes, true))
		{
			$watermarkImages[$fileInfo->getFilename()] = $fileInfo->getFilename();
		}
	}

	sort($watermarkImages);

	return $watermarkImages;
}

/**
 * Adds webp to the allowed types if enabled and available.
 *
 * @param array $allowedTypes
 * @return array
 */
function addWebpType($allowedTypes)
{
	global $modSettings;

	if (!empty($modSettings['attachment_webp_enable']) && in_array('WEBP', Imagick::queryformats(), true))
	{
		$allowedTypes[] = 'webp';
	}

	return $allowedTypes;
}

/**
 * Determines the x/y start coordinate of the watermark based on the 9 available
 * locations.
 *
 * @param int $position
 * @param int $imageWidth
 * @param int $imageHeight
 * @param int $watermarkWidth
 * @param int $watermarkHeight
 * @return array
 */
function watermarkLocation($position, $imageWidth, $imageHeight, $watermarkWidth, $watermarkHeight)
{
	$offset = 10;

	// Quick helpers
	$centerX = round(($imageWidth - $watermarkWidth) / 2);
	$centerY = round(($imageHeight - $watermarkHeight) / 2);
	$top = $offset;
	$left = $offset;
	$right = $imageWidth - $watermarkWidth - $offset;
	$bottom = $imageHeight - $watermarkHeight - $offset;

	switch ($position)
	{
		case 1:
			// Top Left :: Imagick::GRAVITY_NORTHWEST;
			$watermarkX = $left;
			$watermarkY = $top;
			break;
		case 2:
			// Top center :: Imagick::GRAVITY_NORTH;
			$watermarkX = $centerX;
			$watermarkY = $top;
			break;
		case 3:
			// Top Right :: Imagick::GRAVITY_NORTHEAST;
			$watermarkX = $right;
			$watermarkY = $top;
			break;
		case 4:
			// Center left :: Imagick::GRAVITY_WEST;
			$watermarkX = $offset;
			$watermarkY = $centerY;
			break;
		case 5:
			// Center
			$watermarkX = $centerX;
			$watermarkY = $centerY;
			break;
		case 6:
			// Center right :: Imagick::GRAVITY_EAST;
			$watermarkX = $right;
			$watermarkY = $centerY;
			break;
		case 7:
			// Bottom left :: Imagick::GRAVITY_SOUTHWEST;
			$watermarkX = $left;
			$watermarkY = $bottom;
			break;
		case 8:
			// Bottom Center :: Imagick::GRAVITY_SOUTH;
			$watermarkX = $centerX;
			$watermarkY = $bottom;
			break;
		case 9:
			// Bottom right = Imagick::GRAVITY_SOUTHEAST;
			$watermarkX = $right;
			$watermarkY = $bottom;
			break;
		default:
			$watermarkX = 0;
			$watermarkY = 0;
	}

	return array($watermarkX, $watermarkY);
}

/**
 * Checks if this is an image we should watermark
 *
 * @param string $filename
 * @return bool
 */
function validateWatermark($filename)
{
	global $modSettings;

	// Check if this image's WxH justifies the watermark
	$size = elk_getimagesize($filename);

	if ($size[0] === -1)
	{
		return false;
	}

	if (!empty($modSettings['watermarkMinHeight']) && $modSettings['watermarkMinHeight'] > $size[1])
	{
		return false;
	}

	if (!empty($modSettings['watermarkMinWidth']) && $modSettings['watermarkMinWidth'] > $size[0])
	{
		return false;
	}

	// We only watermark these types.
	$allowedTypes = array(IMAGETYPE_JPEG, IMAGETYPE_JPEG2000, IMAGETYPE_PNG, IMAGETYPE_BMP, IMAGETYPE_WBMP,
						  IMAGETYPE_TIFF_II, IMAGETYPE_TIFF_MM);
	if (!empty($modSettings['attachment_webp_enable']) && in_array('WEBP', Imagick::queryformats(), true))
	{
		$allowedTypes[] = IMAGETYPE_WEBP;
	}

	if (!in_array($size[2], $allowedTypes, true))
	{
		return false;
	}

	return true;
}

/**
 * Uses an image as a watermark
 *
 * @param string $sourceImage
 * @param string $watermarkImage
 * @return bool|\Imagick
 */
function watermark($sourceImage, $watermarkImage)
{
	global $modSettings;

	// Load the image to watermark
	try
	{
		$image = new Imagick();
		$image->readImage($sourceImage);

		// Load the watermark
		$watermark = new Imagick();
		$watermark->readImage($watermarkImage);

		// Get all current dimensions
		$imageWidth = $image->getImageWidth();
		$imageHeight = $image->getImageHeight();
		$watermarkWidth = $watermark->getImageWidth();
		$watermarkHeight = $watermark->getImageHeight();

		// Setting a max size or adaptive size for the Watermark?
		setWatermarkScale($watermark, $watermarkHeight, $watermarkWidth, $imageHeight, $imageWidth);

		// Refresh these now
		$watermarkWidth = $watermark->getImageWidth();
		$watermarkHeight = $watermark->getImageHeight();

		// Calculate the overlay position
		$position = empty($modSettings['watermarkPosition']) ? 5 : $modSettings['watermarkPosition'];
		list($x, $y) = watermarkLocation($position, $imageWidth, $imageHeight, $watermarkWidth, $watermarkHeight);

		// Bam! watermarked
		$image->compositeImage($watermark, Imagick::COMPOSITE_OVER, $x, $y);

		return $image;

	}
	catch (Exception $e)
	{
		//var_dump($e->getMessage(), $e->getLine());

		return false;
	}
}

/**
 * Seeming needless complex section of code to proper size the thumbnail based on ACP
 * settings and image constraints.
 *
 * @param Imagick $watermark
 * @param int $watermarkHeight
 * @param int $watermarkWidth
 * @param int $imageHeight
 * @param int $imageWidth
 */
function setWatermarkScale($watermark, $watermarkHeight, $watermarkWidth, $imageHeight, $imageWidth)
{
	global $modSettings;

	$watermarkMaxHeight = empty($modSettings['watermarkScaleHeight']) ? $watermarkHeight : $modSettings['watermarkScaleHeight'];
	$watermarkMaxWidth = empty($modSettings['watermarkScaleWidth']) ? $watermarkWidth : $modSettings['watermarkScaleWidth'];

	if (strpos($watermarkMaxHeight, '%') !== false || strpos($watermarkMaxWidth, '%') !== false)
	{
		// Adaptive
		$watermarkMaxHeight = str_replace('%', '', $watermarkMaxHeight);
		$watermarkMaxWidth = str_replace('%', '', $watermarkMaxWidth);

		$watermarkMaxWidth = (int) round($imageWidth * $watermarkMaxWidth / 100, 0);
		$watermarkMaxHeight = (int) round($imageHeight * $watermarkMaxHeight / 100, 0);
	}
	elseif ($watermarkHeight > $watermarkMaxHeight || $watermarkWidth > $watermarkMaxWidth)
	{
		// Watermark image is larger than the limits?
		$watermarkMaxWidth = min($watermarkWidth, $watermarkMaxWidth);
		$watermarkMaxHeight = min($watermarkHeight, $watermarkMaxHeight);
	}

	// Watermark image is too wide or too tall to fit on the image?
	if ($imageHeight < $watermarkMaxHeight || $imageWidth < $watermarkMaxWidth)
	{
		// Resize the watermark
		$watermarkMaxWidth = min($imageWidth, $watermarkWidth);
		$watermarkMaxHeight = min($imageHeight, $watermarkHeight);
	}

	// Something to do ?
	if ($watermarkMaxWidth !== $watermarkWidth || $watermarkMaxHeight !== $watermarkHeight)
	{
		$watermark->resizeImage($watermarkMaxWidth, $watermarkMaxHeight, Imagick::FILTER_LANCZOS, 1, true);
	}
}

/**
 * Watermarks an image with a line of text
 *
 * @param string $sourceImage
 * @param string $watermarkText
 * @return bool|\Imagick
 */
function watermarkText($sourceImage, $watermarkText = 'Copyright')
{
	global $modSettings;

	$offset = 10;
	$text = empty($modSettings['watermarkText']) ? $watermarkText : $modSettings['watermarkText'];
	$text = html_entity_decode($text);
	$stroke = empty($modSettings['watermarkStrokeSize']) ? 3 : $modSettings['watermarkStrokeSize'];

	try
	{
		$image = new Imagick($sourceImage);
		$watermark = new Imagick();
		$mask = new Imagick();
		$draw = new ImagickDraw();

		// Define dimensions
		$imageWidth = $image->getImageWidth();
		$imageHeight = $image->getImageHeight();

		// Create palettes
		$watermark->newImage($imageWidth, $imageHeight, new ImagickPixel('rgb(40, 40, 40)'));
		$mask->newImage($imageWidth, $imageHeight, new ImagickPixel('black'));

		// Set font properties
		$fontSize = getFontSize($imageWidth, $imageHeight);
		$draw->setFont(BOARDDIR . '/themes/default/fonts/VDS_New.ttf');
		$draw->setFontSize($fontSize);
		$draw->setTextKerning(2.5);

		// Position text
		$position = empty($modSettings['watermarkPosition']) ? 5 : $modSettings['watermarkPosition'];
		$draw->setGravity($position);

		// Draw an outline of the text on the mask palette, grey fill will define the opacity
		// of the text, e.g. 50% of the watermark ImagickPixel color.
		$draw->setFillColor('grey50');
		$stroke = $fontSize < 10 ? 1 : $stroke;
		strokeMe($stroke, $draw, $mask, $text, $offset);

		// Remove the inside of the text
		$draw->setFillColor('black');
		$mask->annotateImage($draw, $offset, $offset, 0, $text);

		// Apply mask to watermark
		$mask->setImageMatte(false);
		$watermark->compositeImage($mask, imagick::COMPOSITE_COPYOPACITY, 0, 0);
		$image->compositeImage($watermark, imagick::COMPOSITE_DISSOLVE, 0, 0);

		return $image;
	}
	catch (Exception $e)
	{
		var_dump($e->getMessage(), $e->getLine());

		return false;
	}
}

/**
 * Stroke me, stroke me, Could be a winner boy you move mighty well.
 * Draws a stroke line around the text, this is what will be visible
 * on the image since we mask out the "inside"
 *
 * @param $stroke
 * @param $draw
 * @param $mask
 * @param $text
 * @param $offset
 */
function strokeMe($stroke, $draw, $mask, $text, $offset)
{
	$ys = 0;
	$xpos = $offset;
	$ypos = $offset;

	$xs = $stroke;

	while ($xs >= $ys)
	{
		$mask->annotateImage($draw, $xpos + $stroke, $ypos + $ys, 0, $text);
		$mask->annotateImage($draw, $xpos + $stroke, $ypos - $ys, 0, $text);
		$mask->annotateImage($draw, $xpos - $stroke, $ypos + $ys, 0, $text);
		$mask->annotateImage($draw, $xpos - $stroke, $ypos - $ys, 0, $text);
		$mask->annotateImage($draw, $xpos + $ys, $ypos + $stroke, 0, $text);
		$mask->annotateImage($draw, $xpos + $ys, $ypos - $stroke, 0, $text);
		$mask->annotateImage($draw, $xpos - $ys, $ypos + $stroke, 0, $text);
		$mask->annotateImage($draw, $xpos - $ys, $ypos - $stroke, 0, $text);

		$ys++;
	}
}

/**
 * Determine the font size to use
 *
 * @param int $imageWidth
 * @param int $imageHeight
 * @return int
 */
function getFontSize($imageWidth, $imageHeight)
{
	global $modSettings;

	$fontSize = empty($modSettings['watermarkTextSize']) ? 80 : $modSettings['watermarkTextSize'];
	if (strpos($fontSize, '%') !== false)
	{
		$fontSize = str_replace('%', '', $fontSize);
		$fontSize = round(max($imageWidth, $imageHeight) * ($fontSize / 100), 0);
		$fontSize = min($fontSize, 1000);
	}

	return (int) $fontSize;
}
